@@ -13,8 +13,6 @@
 import os
 import warnings
 
-from positional import positional
-
 from keystoneauth1 import session
 
 
@@ -27,6 +25,9 @@ class Adapter(object):
     particular client that is using the session. An adapter provides a wrapper
     of client local data around the global session object.
 
+    version, min_version, max_version and default_microversion can all be
+    given either as a string or a tuple.
+
     :param session: The session object to wrap.
     :type session: keystoneauth1.session.Session
     :param str service_type: The default service_type for URL discovery.
@@ -35,7 +36,9 @@ class Adapter(object):
     :param str region_name: The default region_name for URL discovery.
     :param str endpoint_override: Always use this endpoint URL for requests
                                   for this client.
-    :param tuple version: The version that this API targets.
+    :param version: The minimum version restricted to a given Major API.
+                    Mutually exclusive with min_version and max_version.
+                    (optional)
     :param auth: An auth plugin to use instead of the session one.
     :type auth: keystoneauth1.plugin.BaseAuthPlugin
     :param str user_agent: The User-Agent string to set.
@@ -60,18 +63,50 @@ class Adapter(object):
     :param bool allow_version_hack: Allow keystoneauth to hack up catalog
                                     URLS to support older schemes.
                                     (optional, default True)
+    :param str global_request_id: A global_request_id (in the form of
+                                  ``req-$uuid``) that will be passed on all
+                                  requests. Enables cross project request id
+                                  tracking.
+    :param min_version: The minimum major version of a given API, intended to
+                        be used as the lower bound of a range with
+                        max_version. Mutually exclusive with version.
+                        If min_version is given with no max_version it is as
+                        if max version is 'latest'. (optional)
+    :param max_version: The maximum major version of a given API, intended to
+                        be used as the upper bound of a range with min_version.
+                        Mutually exclusive with version. (optional)
+    :param default_microversion: The default microversion value to send
+                                 with API requests. While microversions are
+                                 a per-request feature, a user may know they
+                                 want to default to sending a specific value.
+                                 (optional)
+    :param int status_code_retries: the maximum number of retries that
+                                    should be attempted for retriable
+                                    HTTP status codes (optional, defaults
+                                    to 0 - never retry).
+    :param list retriable_status_codes: list of HTTP status codes that
+                                        should be retried (optional,
+                                        defaults to HTTP 503, has no effect
+                                        when status_code_retries is 0).
     """
 
     client_name = None
     client_version = None
 
-    @positional()
     def __init__(self, session, service_type=None, service_name=None,
                  interface=None, region_name=None, endpoint_override=None,
                  version=None, auth=None, user_agent=None,
-                 connect_retries=None, logger=None, allow={},
+                 connect_retries=None, logger=None, allow=None,
                  additional_headers=None, client_name=None,
-                 client_version=None, allow_version_hack=None):
+                 client_version=None, allow_version_hack=None,
+                 global_request_id=None,
+                 min_version=None, max_version=None,
+                 default_microversion=None, status_code_retries=None,
+                 retriable_status_codes=None):
+        if version and (min_version or max_version):
+            raise TypeError(
+                "version is mutually exclusive with min_version and"
+                " max_version")
         # NOTE(jamielennox): when adding new parameters to adapter please also
         # add them to the adapter call in httpclient.HTTPClient.__init__ as
         # well as to load_adapter_from_argparse below if the argument is
@@ -88,9 +123,16 @@ class Adapter(object):
         self.auth = auth
         self.connect_retries = connect_retries
         self.logger = logger
-        self.allow = allow
+        self.allow = allow or {}
         self.additional_headers = additional_headers or {}
         self.allow_version_hack = allow_version_hack
+        self.min_version = min_version
+        self.max_version = max_version
+        self.default_microversion = default_microversion
+        self.status_code_retries = status_code_retries
+        self.retriable_status_codes = retriable_status_codes
+
+        self.global_request_id = global_request_id
 
         if client_name:
             self.client_name = client_name
@@ -108,6 +150,10 @@ class Adapter(object):
             kwargs.setdefault('region_name', self.region_name)
         if self.version:
             kwargs.setdefault('version', self.version)
+        if self.min_version:
+            kwargs.setdefault('min_version', self.min_version)
+        if self.max_version:
+            kwargs.setdefault('max_version', self.max_version)
         if self.allow_version_hack is not None:
             kwargs.setdefault('allow_version_hack', self.allow_version_hack)
 
@@ -124,10 +170,17 @@ class Adapter(object):
             kwargs.setdefault('user_agent', self.user_agent)
         if self.connect_retries is not None:
             kwargs.setdefault('connect_retries', self.connect_retries)
+        if self.status_code_retries is not None:
+            kwargs.setdefault('status_code_retries', self.status_code_retries)
+        if self.retriable_status_codes:
+            kwargs.setdefault('retriable_status_codes',
+                              self.retriable_status_codes)
         if self.logger:
             kwargs.setdefault('logger', self.logger)
         if self.allow:
             kwargs.setdefault('allow', self.allow)
+        if self.default_microversion is not None:
+            kwargs.setdefault('microversion', self.default_microversion)
 
         if isinstance(self.session, (session.Session, Adapter)):
             # these things are unsupported by keystoneclient's session so be
@@ -145,6 +198,10 @@ class Adapter(object):
         for k, v in self.additional_headers.items():
             kwargs.setdefault('headers', {}).setdefault(k, v)
 
+        if self.global_request_id is not None:
+            kwargs.setdefault('headers', {}).setdefault(
+                "X-OpenStack-Request-ID", self.global_request_id)
+
         return self.session.request(url, method, **kwargs)
 
     def get_token(self, auth=None):
@@ -181,6 +238,46 @@ class Adapter(object):
         self._set_endpoint_filter_kwargs(kwargs)
         return self.session.get_endpoint(auth or self.auth, **kwargs)
 
+    def get_endpoint_data(self, auth=None):
+        """Get the endpoint data for this Adapter's endpoint.
+
+        :param auth: The auth plugin to use for token. Overrides the plugin on
+                     the session. (optional)
+        :type auth: keystoneauth1.plugin.BaseAuthPlugin
+
+        :raises keystoneauth1.exceptions.auth_plugins.MissingAuthPlugin: if a
+            plugin is not available.
+        :raises TypeError: If arguments are invalid
+
+        :returns: Endpoint data if available or None.
+        :rtype: keystoneauth1.discover.EndpointData
+        """
+        kwargs = {}
+        self._set_endpoint_filter_kwargs(kwargs)
+        if self.endpoint_override:
+            kwargs['endpoint_override'] = self.endpoint_override
+
+        return self.session.get_endpoint_data(auth or self.auth, **kwargs)
+
+    def get_api_major_version(self, auth=None, **kwargs):
+        """Get the major API version as provided by the auth plugin.
+
+        :param auth: The auth plugin to use for token. Overrides the plugin on
+                     the session. (optional)
+        :type auth: keystoneauth1.plugin.BaseAuthPlugin
+
+        :raises keystoneauth1.exceptions.auth_plugins.MissingAuthPlugin: if a
+            plugin is not available.
+
+        :return: The major version of the API of the service discovered.
+        :rtype: tuple or None
+        """
+        self._set_endpoint_filter_kwargs(kwargs)
+        if self.endpoint_override:
+            kwargs['endpoint_override'] = self.endpoint_override
+
+        return self.session.get_api_major_version(auth or self.auth, **kwargs)
+
     def invalidate(self, auth=None):
         """Invalidate an authentication plugin."""
         return self.session.invalidate(auth or self.auth)
